'use strict';

var _ = require('lodash');
var transformDwayneHtml = require('transform-dwayne-html');

var _require = require('lines-and-columns'),
    LinesAndColumns = _require.default;

module.exports = function (_ref) {
  var t = _ref.types;
  return {
    visitor: {
      Program: function Program(path, state) {
        state.file.__dwayneLines__ = new LinesAndColumns(state.file.code);
      },
      TaggedTemplateExpression: function TaggedTemplateExpression(path, state) {
        var options = state.opts,
            _state$file = state.file,
            __dwayneLines__ = _state$file.__dwayneLines__,
            code = _state$file.code,
            filename = _state$file.log.filename;


        options.unscopables = _.get(options, 'unscopables', ['require']);
        options.useES6 = !!_.get(options, 'useES6', true);

        var taggedHtmlFuncName = _.get(options, 'taggedHtmlFuncName', 'html');
        var taggedHtmlScopelessFuncName = _.get(options, 'taggedHtmlScopelessFuncName', 'htmlScopeless');
        var _path$node = path.node,
            _path$node$quasi = _path$node.quasi,
            start = _path$node$quasi.start,
            quasis = _path$node$quasi.quasis,
            expressions = _path$node$quasi.expressions,
            name = _path$node.tag.name;


        if (quasis.length !== 1 || expressions.length || name !== taggedHtmlFuncName && name !== taggedHtmlScopelessFuncName) {
          return;
        }

        var isScopeless = name === taggedHtmlScopelessFuncName;
        var tmplVarName = generateScopeVar('_tmpl', path, options);
        var mixinVarName = generateScopeVar('_mixin', path, options);
        var thisVarName = isScopeless && generateScopeVar('_this', path, options);
        var lines = __dwayneLines__ || /* istanbul ignore next */new LinesAndColumns(code);
        var loc = lines.locationForIndex(start + 1);
        var transformerOpts = _.assign({}, options, {
          filename: filename,
          sourceType: 'embed',
          inputSourceMap: null,
          startLine: loc.line + 1,
          startColumn: loc.column,
          startPosition: start + 1,
          tmplVarName: tmplVarName,
          mixinVarName: mixinVarName,
          thisVarName: thisVarName,
          keepScope: isScopeless
        });
        var transformed = transformDwayneHtml(quasis[0].value.cooked, transformerOpts);

        path.replaceWithSourceString(transformed.code);

        if (transformed.generatedTmplVar) {
          path.scope.push({
            id: t.identifier(tmplVarName),
            kind: options.useES6 ? 'let' : 'var'
          });
        }

        if (transformed.generatedMixinVar) {
          path.scope.push({
            id: t.identifier(mixinVarName),
            kind: options.useES6 ? 'let' : 'var'
          });
        }

        if (transformed.generatedThisVar) {
          path.scope.push({
            id: t.identifier(thisVarName),
            init: t.thisExpression(),
            kind: 'var'
          });
        }
      }
    }
  };
};

function generateScopeVar(name, path, options) {
  var varName = void 0;

  do {
    varName = path.scope.generateUid(name);
  } while (options.unscopables.indexOf(varName) !== -1);

  return varName;
}